@@ -309,9 +309,15 @@ static inline int _mk_event_timeout_create(struct mk_event_ctx *ctx,
 
 static inline int _mk_event_timeout_destroy(struct mk_event_ctx *ctx, void *data)
 {
-    (void) ctx;
-    (void) data;
+    struct mk_event *event;
 
+    if (!data) {
+        return 0;
+    }
+
+    event = (struct mk_event *) data;
+    _mk_event_del(ctx, event);
+    close(event->fd);
     return 0;
 }
 
