@@ -75,7 +75,19 @@ struct flb_kube *flb_kube_conf_create(struct flb_filter_instance *ins,
 
     /* Get Kubernetes API server */
     url = flb_filter_get_property("kube_url", ins);
-    if (!url) {
+
+    if (ctx->use_kubelet) {
+        ctx->api_host = flb_strdup(FLB_KUBELET_HOST);
+        ctx->api_port = ctx->kubelet_port;
+        ctx->api_https = FLB_TRUE;
+
+        /* This is for unit test diagnostic purposes */
+        if (ctx->meta_preload_cache_dir) {
+            ctx->api_https = FLB_FALSE;
+        }
+
+    }
+    else if (!url) {
         ctx->api_host = flb_strdup(FLB_API_HOST);
         ctx->api_port = FLB_API_PORT;
         ctx->api_https = FLB_API_TLS;
