@@ -146,34 +146,24 @@ static int get_local_pod_info(struct flb_kube *ctx)
     return FLB_TRUE;
 }
 
-/* Gather metadata from API Server */
-static int get_api_server_info(struct flb_kube *ctx,
-                               const char *namespace, const char *podname,
-                               char **out_buf, size_t *out_size)
-{
-    int ret;
+/*
+ * If a file exists called namespace_podname.meta, load it and use it.
+ * If not, fall back to API. This is primarily for diagnostic purposes,
+ * e.g. debugging new parsers.
+ */
+static int get_meta_file_info(struct flb_kube *ctx, const char *namespace,
+                              const char *podname, char **buffer, size_t *size,
+                              int *root_type) {
+
+    int fd = -1;
+    char *payload = NULL;
+    size_t payload_size = 0;
+    struct stat sb;
     int packed = -1;
-    int root_type;
-    size_t b_sent;
+    int ret;
     char uri[1024];
-    char *buf;
-    size_t size;
-    struct flb_http_client *c;
-    struct flb_upstream_conn *u_conn;
 
-    *out_buf = NULL;
-    *out_size = 0;
-
-    /*
-     * If a file exists called namespace_podname.meta, load it and use it.
-     * If not, fall back to API. This is primarily for diagnostic purposes,
-     * e.g. debugging new parsers.
-     */
     if (ctx->meta_preload_cache_dir && namespace && podname) {
-        int fd = -1;
-        char *payload = NULL;
-        size_t payload_size = 0;
-        struct stat sb;
 
         ret = snprintf(uri, sizeof(uri) - 1, "%s/%s_%s.meta",
                        ctx->meta_preload_cache_dir, namespace, podname);
@@ -198,7 +188,7 @@ static int get_api_server_info(struct flb_kube *ctx,
 
         if (payload_size) {
             packed = flb_pack_json(payload, payload_size,
-                                   &buf, &size, &root_type);
+                                   buffer, size, root_type);
         }
 
         if (payload) {
@@ -206,58 +196,147 @@ static int get_api_server_info(struct flb_kube *ctx,
         }
     }
 
-    if (packed == -1) {
-        if (!ctx->upstream) {
-            return -1;
+    return packed;
+}
+
+/* Gather metadata from HTTP Request,
+ * this could send out HTTP Request either to KUBE Server API or Kubelet
+ */
+static int get_meta_info_from_request(struct flb_kube *ctx,
+                                      const char *namespace,
+                                      const char *podname,
+                                      char **buffer, size_t *size,
+                                      int *root_type,
+                                      char* uri)
+{
+    struct flb_http_client *c;
+    struct flb_upstream_conn *u_conn;
+    int ret;
+    size_t b_sent;
+    int packed;
+
+    if (!ctx->upstream) {
+        return -1;
+    }
+
+    u_conn = flb_upstream_conn_get(ctx->upstream);
+
+    if (!u_conn) {
+        flb_plg_error(ctx->ins, "kubelet upstream connection error");
+        return -1;
+    }
+
+    /* Compose HTTP Client request*/
+    c = flb_http_client(u_conn, FLB_HTTP_GET,
+                        uri,
+                        NULL, 0, NULL, 0, NULL, 0);
+    flb_http_buffer_size(c, ctx->buffer_size);
+
+    flb_http_add_header(c, "User-Agent", 10, "Fluent-Bit", 10);
+    flb_http_add_header(c, "Connection", 10, "close", 5);
+    if (ctx->auth_len > 0) {
+        flb_http_add_header(c, "Authorization", 13, ctx->auth, ctx->auth_len);
+    }
+
+    ret = flb_http_do(c, &b_sent);
+    flb_plg_debug(ctx->ins, "Request (ns=%s, pod=%s) http_do=%i, "
+                  "HTTP Status: %i",
+                  namespace, podname, ret, c->resp.status);
+
+    if (ret != 0 || c->resp.status != 200) {
+        if (c->resp.payload_size > 0) {
+            flb_plg_debug(ctx->ins, "HTTP response\n%s",
+                          c->resp.payload);
         }
-
-        u_conn = flb_upstream_conn_get(ctx->upstream);
-        if (!u_conn) {
-            flb_plg_error(ctx->ins, "upstream connection error");
-            return -1;
-        }
-
-        ret = snprintf(uri, sizeof(uri) - 1,
-                       FLB_KUBE_API_FMT,
-                       namespace, podname);
-        if (ret == -1) {
-            flb_upstream_conn_release(u_conn);
-            return -1;
-        }
-
-        /* Compose HTTP Client request */
-        c = flb_http_client(u_conn, FLB_HTTP_GET,
-                            uri,
-                            NULL, 0, NULL, 0, NULL, 0);
-        flb_http_buffer_size(c, ctx->buffer_size);
-
-        flb_http_add_header(c, "User-Agent", 10, "Fluent-Bit", 10);
-        flb_http_add_header(c, "Connection", 10, "close", 5);
-        if (ctx->auth_len > 0) {
-            flb_http_add_header(c, "Authorization", 13, ctx->auth, ctx->auth_len);
-        }
-
-        /* Perform request */
-        ret = flb_http_do(c, &b_sent);
-        flb_plg_debug(ctx->ins, "API Server (ns=%s, pod=%s) http_do=%i, "
-                      "HTTP Status: %i",
-                      namespace, podname, ret, c->resp.status);
-
-        if (ret != 0 || c->resp.status != 200) {
-            if (c->resp.payload_size > 0) {
-                flb_plg_debug(ctx->ins, "API Server response\n%s",
-                              c->resp.payload);
-            }
-            flb_http_client_destroy(c);
-            flb_upstream_conn_release(u_conn);
-            return -1;
-        }
-        packed = flb_pack_json(c->resp.payload, c->resp.payload_size,
-                               &buf, &size, &root_type);
-
-        /* release resources */
         flb_http_client_destroy(c);
         flb_upstream_conn_release(u_conn);
+        return -1;
+    }
+
+    packed = flb_pack_json(c->resp.payload, c->resp.payload_size,
+                                   buffer, size, root_type);
+
+    /* release resources */
+    flb_http_client_destroy(c);
+    flb_upstream_conn_release(u_conn);
+
+    return packed;
+
+}
+
+/* Gather pods list information from Kubelet */
+static int get_pods_from_kubelet(struct flb_kube *ctx,
+                                 const char *namespace, const char *podname,
+                                 char **out_buf, size_t *out_size)
+{
+    int ret;
+    int packed = -1;
+    int root_type;
+    char uri[1024];
+    char *buf;
+    size_t size;
+
+    *out_buf = NULL;
+    *out_size = 0;
+
+    /* used for unit test purposes*/
+    packed = get_meta_file_info(ctx, namespace, podname, &buf, &size,
+                                &root_type);
+
+    if (packed == -1) {
+
+        ret = snprintf(uri, sizeof(uri) - 1, FLB_KUBELET_PODS);
+        if (ret == -1) {
+            return -1;
+        }
+        flb_plg_debug(ctx->ins,
+                      "Send out request to Kubelet for pods information.");
+        packed = get_meta_info_from_request(ctx, namespace, podname,
+                                            &buf, &size, &root_type, uri);
+    }
+
+    /* validate pack */
+    if (packed == -1) {
+        return -1;
+    }
+
+    *out_buf = buf;
+    *out_size = size;
+
+    return 0;
+}
+
+/* Gather metadata from API Server */
+static int get_api_server_info(struct flb_kube *ctx,
+                               const char *namespace, const char *podname,
+                               char **out_buf, size_t *out_size)
+{
+    int ret;
+    int packed = -1;
+    int root_type;
+    char uri[1024];
+    char *buf;
+    size_t size;
+
+    *out_buf = NULL;
+    *out_size = 0;
+
+    /* used for unit test purposes*/
+    packed = get_meta_file_info(ctx, namespace, podname,
+                                &buf, &size, &root_type);
+
+    if (packed == -1) {
+
+        ret = snprintf(uri, sizeof(uri) - 1, FLB_KUBE_API_FMT, namespace,
+                       podname);
+
+        if (ret == -1) {
+            return -1;
+        }
+        flb_plg_debug(ctx->ins,
+                      "Send out request to API Server for pods information");
+        packed = get_meta_info_from_request(ctx, namespace, podname,
+                                            &buf, &size, &root_type, uri);
     }
 
     /* validate pack */
@@ -477,6 +556,147 @@ static void extract_container_hash(struct flb_kube_meta *meta,
     }
 }
 
+static int search_podname_and_namespace(struct flb_kube_meta *meta,
+                                        struct flb_kube *ctx,
+                                        msgpack_object map)
+{
+    int i;
+    int podname_found = FLB_FALSE;
+    int namespace_found = FLB_FALSE;
+    int target_podname_found = FLB_FALSE;
+    int target_namespace_found = FLB_FALSE;
+
+    msgpack_object k;
+    msgpack_object v;
+
+    for (i = 0; (!podname_found || !namespace_found) &&
+                 i < map.via.map.size; i++) {
+
+        k = map.via.map.ptr[i].key;
+        v = map.via.map.ptr[i].val;
+        if (k.via.str.size == 4 && !strncmp(k.via.str.ptr, "name", 4)) {
+
+            podname_found = FLB_TRUE;
+            if (!strncmp(v.via.str.ptr, meta->podname, meta->podname_len)) {
+                target_podname_found = FLB_TRUE;
+            }
+
+        }
+        else if (k.via.str.size == 9 && !strncmp(k.via.str.ptr,
+                                                 "namespace", 9)) {
+
+            namespace_found = FLB_TRUE;
+            if (!strncmp((char *)v.via.str.ptr,
+                          meta->namespace,
+                          meta->namespace_len)) {
+                target_namespace_found = FLB_TRUE;
+            }
+        }
+    }
+
+    if (!target_podname_found || !target_namespace_found) {
+        return -1;
+    }
+
+    return 0;
+}
+
+static int search_metadata_in_items(struct flb_kube_meta *meta,
+                                    struct flb_kube *ctx,
+                                    msgpack_object items_array,
+                                    msgpack_object *target_item_map)
+{
+    int i, j;
+
+    int target_found = FLB_FALSE;
+    msgpack_object item_info_map;
+    msgpack_object k;
+    msgpack_object v;
+
+    for (i = 0; !target_found && i < items_array.via.array.size; i++) {
+
+        item_info_map = items_array.via.array.ptr[i];
+        if (item_info_map.type != MSGPACK_OBJECT_MAP) {
+            continue;
+        }
+
+        for (j = 0; j < item_info_map.via.map.size; j++) {
+
+            k = item_info_map.via.map.ptr[j].key;
+            if (k.via.str.size == 8 &&
+                !strncmp(k.via.str.ptr, "metadata", 8)) {
+
+                v = item_info_map.via.map.ptr[j].val;
+                if (search_podname_and_namespace(meta, ctx, v) == 0) {
+                    target_found = FLB_TRUE;
+                    *target_item_map = item_info_map;
+                    flb_plg_debug(ctx->ins,
+                                  "kubelet find pod: %s and ns: %s match",
+                                            meta->podname, meta->namespace);
+                }
+                break;
+            }
+        }
+    }
+
+    if (!target_found) {
+        flb_plg_debug(ctx->ins,
+                      "kubelet didn't find pod: %s, ns: %s match",
+                      meta->podname, meta->namespace);
+        return -1;
+    }
+    return 0;
+}
+
+/* At this point map points to the ROOT map, eg:
+ *
+ * {
+ *  "kind": "PodList",
+ *  "apiVersion": "v1",
+ *  "metadata": {},
+ *  "items": [{
+ *    "metadata": {
+ *      "name": "fluent-bit-rz47v",
+ *      "generateName": "fluent-bit-",
+ *      "namespace": "kube-system",
+ *      "selfLink": "/api/v1/namespaces/kube-system/pods/fluent-bit-rz47v",
+ *      ....
+ *    }
+ *   }]
+ *
+ */
+static int search_item_in_items(struct flb_kube_meta *meta,
+                                struct flb_kube *ctx,
+                                msgpack_object api_map,
+                                msgpack_object *target_item_map)
+{
+
+    int i;
+    int items_array_found = FLB_FALSE;
+
+    msgpack_object k;
+    msgpack_object v;
+    msgpack_object items_array;
+
+    for (i = 0; !items_array_found && i < api_map.via.map.size; i++) {
+
+        k = api_map.via.map.ptr[i].key;
+        if (k.via.str.size == 5 && !strncmp(k.via.str.ptr, "items", 5)) {
+
+            v = api_map.via.map.ptr[i].val;
+            if (v.type == MSGPACK_OBJECT_ARRAY) {
+                items_array = v;
+                items_array_found = FLB_TRUE;
+            }
+        }
+    }
+
+    int ret = search_metadata_in_items(meta, ctx, items_array,
+                                       target_item_map);
+
+    return ret;
+}
+
 static int merge_meta(struct flb_kube_meta *meta, struct flb_kube *ctx,
                       const char *api_buf, size_t api_size,
                       char **out_buf, size_t *out_size)
@@ -487,6 +707,7 @@ static int merge_meta(struct flb_kube_meta *meta, struct flb_kube *ctx,
     int meta_found = FLB_FALSE;
     int spec_found = FLB_FALSE;
     int status_found = FLB_FALSE;
+    int target_found = FLB_TRUE;
     int have_uid = -1;
     int have_labels = -1;
     int have_annotations = -1;
@@ -497,6 +718,7 @@ static int merge_meta(struct flb_kube_meta *meta, struct flb_kube *ctx,
 
     msgpack_unpacked api_result;
     msgpack_unpacked meta_result;
+    msgpack_object item_result;
     msgpack_object k;
     msgpack_object v;
     msgpack_object meta_val;
@@ -528,7 +750,16 @@ static int merge_meta(struct flb_kube_meta *meta, struct flb_kube *ctx,
         msgpack_unpacked_init(&api_result);
         ret = msgpack_unpack_next(&api_result, api_buf, api_size, &off);
         if (ret == MSGPACK_UNPACK_SUCCESS) {
-            api_map = api_result.data;
+
+            if (ctx->use_kubelet) {
+                ret = search_item_in_items(meta, ctx, api_result.data, &item_result);
+                if (ret == -1) {
+                    target_found = FLB_FALSE;
+                }
+                api_map = target_found ? item_result : api_result.data;
+            } else  {
+                api_map = api_result.data;
+            }
 
             /* At this point map points to the ROOT map, eg:
              *
@@ -547,7 +778,7 @@ static int merge_meta(struct flb_kube_meta *meta, struct flb_kube *ctx,
              * We are also interested in the spec.nodeName.
              * We are also interested in the status.containerStatuses.
              */
-            for (i = 0; !(meta_found && spec_found && status_found) &&
+            for (i = 0; target_found && !(meta_found && spec_found && status_found) &&
                         i < api_map.via.map.size; i++) {
                 k = api_map.via.map.ptr[i].key;
                 if (k.via.str.size == 8 && !strncmp(k.via.str.ptr, "metadata", 8)) {
@@ -898,9 +1129,13 @@ static int get_and_merge_meta(struct flb_kube *ctx, struct flb_kube_meta *meta,
     char *api_buf;
     size_t api_size;
 
-    ret = get_api_server_info(ctx,
-                              meta->namespace, meta->podname,
-                              &api_buf, &api_size);
+    if (ctx->use_kubelet) {
+        ret = get_pods_from_kubelet(ctx, meta->namespace, meta->podname,
+                                    &api_buf, &api_size);
+    } else {
+        ret = get_api_server_info(ctx, meta->namespace, meta->podname,
+                                  &api_buf, &api_size);
+    }
     if (ret == -1) {
         return -1;
     }
@@ -935,8 +1170,8 @@ static int wait_for_dns(struct flb_kube *ctx)
             freeaddrinfo(res);
             return 0;
         }
-        flb_plg_info(ctx->ins, "Wait %i secs until DNS starts up (%i/%i)",
-                     ctx->dns_wait_time, i + 1, ctx->dns_retries);
+        flb_plg_info(ctx->ins, "host: %s Wait %i secs until DNS starts up (%i/%i)",
+                     ctx->api_host, ctx->dns_wait_time, i + 1, ctx->dns_retries);
         sleep(ctx->dns_wait_time);
     }
     return -1;
@@ -961,6 +1196,7 @@ static int flb_kube_network_init(struct flb_kube *ctx, struct flb_config *config
         if (!ctx->tls) {
             return -1;
         }
+
         io_type = FLB_IO_TLS;
     }
 
@@ -972,6 +1208,7 @@ static int flb_kube_network_init(struct flb_kube *ctx, struct flb_config *config
                                         ctx->tls);
     if (!ctx->upstream) {
         /* note: if ctx->tls.context is set, it's destroyed upon context exit */
+        flb_plg_debug(ctx->ins, "kube network init create upstream failed");
         return -1;
     }
 
@@ -1001,17 +1238,24 @@ int flb_kube_meta_init(struct flb_kube *ctx, struct flb_config *config)
     if (ret == FLB_TRUE) {
         flb_plg_info(ctx->ins, "local POD info OK");
 
-        /* Gather info from API server */
-        flb_plg_info(ctx->ins, "testing connectivity with API server...");
-
         ret = wait_for_dns(ctx);
         if (ret == -1) {
             flb_plg_warn(ctx->ins, "could not resolve %s", ctx->api_host);
             return -1;
         }
 
-        ret = get_api_server_info(ctx, ctx->namespace, ctx->podname,
-                                  &meta_buf, &meta_size);
+        if (ctx->use_kubelet) {
+            /* Gather info from Kubelet */
+            flb_plg_info(ctx->ins, "testing connectivity with Kubelet...");
+            ret = get_pods_from_kubelet(ctx, ctx->namespace, ctx->podname,
+                                              &meta_buf, &meta_size);
+        }
+        else {
+            /* Gather info from API server */
+            flb_plg_info(ctx->ins, "testing connectivity with API server...");
+            ret = get_api_server_info(ctx, ctx->namespace, ctx->podname,
+                                   &meta_buf, &meta_size);
+        }
         if (ret == -1) {
             if (!ctx->podname) {
                 flb_plg_warn(ctx->ins, "could not get meta for local POD");
@@ -1022,7 +1266,7 @@ int flb_kube_meta_init(struct flb_kube *ctx, struct flb_config *config)
             }
             return -1;
         }
-        flb_plg_info(ctx->ins, "API server connectivity OK");
+        flb_plg_info(ctx->ins, "connectivity OK");
         flb_free(meta_buf);
     }
     else {
