@@ -2,7 +2,7 @@
 
 /*  Fluent Bit
  *  ==========
- *  Copyright (C) 2019-2020 The Fluent Bit Authors
+ *  Copyright (C) 2019-2021 The Fluent Bit Authors
  *  Copyright (C) 2015-2018 Treasure Data Inc.
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
@@ -37,6 +37,16 @@ static void flb_loki_kv_init(struct mk_list *list)
     mk_list_init(list);
 }
 
+static inline void safe_sds_cat(flb_sds_t *buf, const char *str, int len)
+{
+    flb_sds_t tmp;
+
+    tmp = flb_sds_cat(*buf, str, len);
+    if (tmp) {
+        *buf = tmp;
+    }
+}
+
 static inline void normalize_cat(struct flb_ra_parser *rp, flb_sds_t name)
 {
     int sub;
@@ -49,13 +59,12 @@ static inline void normalize_cat(struct flb_ra_parser *rp, flb_sds_t name)
     /* Iterate record accessor keys */
     key = rp->key;
     if (rp->type == FLB_RA_PARSER_STRING) {
-        flb_sds_cat(name, key->name, flb_sds_len(key->name));
+        safe_sds_cat(&name, key->name, flb_sds_len(key->name));
     }
     else if (rp->type == FLB_RA_PARSER_KEYMAP) {
-        flb_sds_cat(name, key->name, flb_sds_len(key->name));
-
+        safe_sds_cat(&name, key->name, flb_sds_len(key->name));
         if (mk_list_size(key->subkeys) > 0) {
-            flb_sds_cat(name, "_", 1);
+            safe_sds_cat(&name, "_", 1);
         }
 
         sub = 0;
@@ -63,15 +72,15 @@ static inline void normalize_cat(struct flb_ra_parser *rp, flb_sds_t name)
             entry = mk_list_entry(s_head, struct flb_ra_subentry, _head);
 
             if (sub > 0) {
-                flb_sds_cat(name, "_", 1);
+                safe_sds_cat(&name, "_", 1);
             }
             if (entry->type == FLB_RA_PARSER_STRING) {
-                flb_sds_cat(name, entry->str, flb_sds_len(entry->str));
+                safe_sds_cat(&name, entry->str, flb_sds_len(entry->str));
             }
             else if (entry->type == FLB_RA_PARSER_ARRAY_ID) {
                 len = snprintf(tmp, sizeof(tmp) -1, "%d",
                                entry->array_id);
-                flb_sds_cat(name, tmp, len);
+                safe_sds_cat(&name, tmp, len);
             }
             sub++;
         }
@@ -103,10 +112,33 @@ static flb_sds_t normalize_ra_key_name(struct flb_loki *ctx,
     return name;
 }
 
+void flb_loki_kv_destroy(struct flb_loki_kv *kv)
+{
+    /* destroy key and value */
+    flb_sds_destroy(kv->key);
+    if (kv->val_type == FLB_LOKI_KV_STR) {
+        flb_sds_destroy(kv->str_val);
+    }
+    else if (kv->val_type == FLB_LOKI_KV_RA) {
+        flb_ra_destroy(kv->ra_val);
+    }
+
+    if (kv->ra_key) {
+        flb_ra_destroy(kv->ra_key);
+    }
+
+    if (kv->key_normalized) {
+        flb_sds_destroy(kv->key_normalized);
+    }
+
+    flb_free(kv);
+}
+
 int flb_loki_kv_append(struct flb_loki *ctx, char *key, char *val)
 {
     int ra_count = 0;
     int k_len;
+    int ret;
     struct flb_loki_kv *kv;
 
     if (!key) {
@@ -128,6 +160,7 @@ int flb_loki_kv_append(struct flb_loki *ctx, char *key, char *val)
         flb_plg_error(ctx->ins,
                       "key name for record accessor cannot start with a number: %s",
                       key);
+        flb_free(kv);
         return -1;
     }
 
@@ -147,8 +180,7 @@ int flb_loki_kv_append(struct flb_loki *ctx, char *key, char *val)
             flb_plg_error(ctx->ins,
                           "invalid key record accessor pattern for key '%s'",
                           key);
-            flb_sds_destroy(kv->key);
-            flb_free(kv);
+            flb_loki_kv_destroy(kv);
             return -1;
         }
 
@@ -158,8 +190,12 @@ int flb_loki_kv_append(struct flb_loki *ctx, char *key, char *val)
             flb_plg_error(ctx->ins,
                           "could not normalize key pattern name '%s'\n",
                           kv->ra_key->pattern);
-            flb_sds_destroy(kv->key);
-            flb_free(kv);
+            flb_loki_kv_destroy(kv);
+            return -1;
+        }
+        ret = flb_slist_add(&ctx->remove_keys_derived, key);
+        if (ret < 0) {
+            flb_loki_kv_destroy(kv);
             return -1;
         }
         ra_count++;
@@ -172,8 +208,12 @@ int flb_loki_kv_append(struct flb_loki *ctx, char *key, char *val)
             flb_plg_error(ctx->ins,
                           "invalid record accessor pattern for key '%s': %s",
                           key, val);
-            flb_sds_destroy(kv->key);
-            flb_free(kv);
+            flb_loki_kv_destroy(kv);
+            return -1;
+        }
+        ret = flb_slist_add(&ctx->remove_keys_derived, val);
+        if (ret < 0) {
+            flb_loki_kv_destroy(kv);
             return -1;
         }
         ra_count++;
@@ -182,8 +222,7 @@ int flb_loki_kv_append(struct flb_loki *ctx, char *key, char *val)
         kv->val_type = FLB_LOKI_KV_STR;
         kv->str_val = flb_sds_create(val);
         if (!kv->str_val) {
-            flb_sds_destroy(kv->key);
-            flb_free(kv);
+            flb_loki_kv_destroy(kv);
             return -1;
         }
     }
@@ -202,27 +241,9 @@ static void flb_loki_kv_exit(struct flb_loki *ctx)
     mk_list_foreach_safe(head, tmp, &ctx->labels_list) {
         kv = mk_list_entry(head, struct flb_loki_kv, _head);
 
-        /* unlink */
+        /* unlink and destroy */
         mk_list_del(&kv->_head);
-
-        /* destroy key and value */
-        flb_sds_destroy(kv->key);
-        if (kv->val_type == FLB_LOKI_KV_STR) {
-            flb_sds_destroy(kv->str_val);
-        }
-        else if (kv->val_type == FLB_LOKI_KV_RA) {
-            flb_ra_destroy(kv->ra_val);
-        }
-
-        if (kv->ra_key) {
-            flb_ra_destroy(kv->ra_key);
-        }
-
-        if (kv->key_normalized) {
-            flb_sds_destroy(kv->key_normalized);
-        }
-
-        flb_free(kv);
+        flb_loki_kv_destroy(kv);
     }
 }
 
@@ -365,6 +386,9 @@ static flb_sds_t pack_labels(struct flb_loki *ctx,
                 msgpack_pack_str(mp_pck, v.via.str.size);
                 msgpack_pack_str_body(mp_pck, v.via.str.ptr,  v.via.str.size);
             }
+        }
+
+        if (rval) {
             flb_ra_key_value_destroy(rval);
         }
     }
@@ -484,6 +508,88 @@ static int parse_labels(struct flb_loki *ctx)
     return 0;
 }
 
+static int key_is_duplicated(struct mk_list *list, char *str, int len)
+{
+    struct mk_list *head;
+    struct flb_slist_entry *entry;
+
+    mk_list_foreach(head, list) {
+        entry = mk_list_entry(head, struct flb_slist_entry, _head);
+        if (flb_sds_len(entry->str) == len &&
+            strncmp(entry->str, str, len) == 0) {
+            return FLB_TRUE;
+        }
+    }
+
+    return FLB_FALSE;
+}
+
+static int prepare_remove_keys(struct flb_loki *ctx)
+{
+    int ret;
+    int len;
+    int size;
+    char *tmp;
+    struct mk_list *head;
+    struct flb_slist_entry *entry;
+    struct mk_list *patterns;
+
+    patterns = &ctx->remove_keys_derived;
+
+    /* Add remove keys set in the configuration */
+    if (ctx->remove_keys) {
+        mk_list_foreach(head, ctx->remove_keys) {
+            entry = mk_list_entry(head, struct flb_slist_entry, _head);
+
+            if (entry->str[0] != '$') {
+                tmp = flb_malloc(flb_sds_len(entry->str) + 2);
+                if (!tmp) {
+                    flb_errno();
+                    continue;
+                }
+                else {
+                    tmp[0] = '$';
+                    len = flb_sds_len(entry->str);
+                    memcpy(tmp + 1, entry->str, len);
+                    tmp[len + 1] = '\0';
+                    len++;
+                }
+            }
+            else {
+                tmp = entry->str;
+                len = flb_sds_len(entry->str);
+            }
+
+            ret = key_is_duplicated(patterns, tmp, len);
+            if (ret == FLB_TRUE) {
+                if (entry->str != tmp) {
+                    flb_free(tmp);
+                }
+                continue;
+            }
+
+            ret = flb_slist_add_n(patterns, tmp, len);
+            if (entry->str != tmp) {
+                flb_free(tmp);
+            }
+            if (ret < 0) {
+                return -1;
+            }
+        }
+    }
+
+    size = mk_list_size(patterns);
+    flb_plg_debug(ctx->ins, "remove_mpa size: %d", size);
+    if (size > 0) {
+        ctx->remove_mpa = flb_mp_accessor_create(patterns);
+        if (ctx->remove_mpa == NULL) {
+            return -1;
+        }
+    }
+
+    return 0;
+}
+
 static void loki_config_destroy(struct flb_loki *ctx)
 {
     if (ctx->u) {
@@ -493,6 +599,12 @@ static void loki_config_destroy(struct flb_loki *ctx)
     if (ctx->ra_k8s) {
         flb_ra_destroy(ctx->ra_k8s);
     }
+
+    if (ctx->remove_mpa) {
+        flb_mp_accessor_destroy(ctx->remove_mpa);
+    }
+    flb_slist_destroy(&ctx->remove_keys_derived);
+
     flb_loki_kv_exit(ctx);
     flb_free(ctx);
 }
@@ -526,12 +638,21 @@ static struct flb_loki *loki_config_create(struct flb_output_instance *ins,
         return NULL;
     }
 
+    /* Initialize final remove_keys list */
+    flb_slist_create(&ctx->remove_keys_derived);
+
     /* Parse labels */
     ret = parse_labels(ctx);
     if (ret == -1) {
         return NULL;
     }
 
+    /* Load remove keys */
+    ret = prepare_remove_keys(ctx);
+    if (ret == -1) {
+        return NULL;
+    }
+
     /* Line Format */
     if (strcasecmp(ctx->line_format, "json") == 0) {
         ctx->out_line_format = FLB_LOKI_FMT_JSON;
@@ -562,11 +683,12 @@ static struct flb_loki *loki_config_create(struct flb_output_instance *ins,
                                    ins->host.name,
                                    ins->host.port,
                                    io_flags,
-                                   &ins->tls);
+                                   ins->tls);
     if (!upstream) {
         return NULL;
     }
     ctx->u = upstream;
+    flb_output_upstream_set(ctx->u, ins);
     ctx->tcp_port = ins->host.port;
     ctx->tcp_host = ins->host.name;
 
@@ -594,15 +716,6 @@ static void pack_timestamp(msgpack_packer *mp_pck, struct flb_time *tms)
     msgpack_pack_str_body(mp_pck, buf, len);
 }
 
-static inline void safe_sds_cat(flb_sds_t *buf, const char *str, int len)
-{
-    flb_sds_t tmp;
-
-    tmp = flb_sds_cat(*buf, str, len);
-    if (tmp) {
-        *buf = tmp;
-    }
-}
 
 static void pack_format_line_value(flb_sds_t buf, msgpack_object *val)
 {
@@ -683,21 +796,46 @@ static void pack_format_line_value(flb_sds_t buf, msgpack_object *val)
     }
 }
 
-
 static int pack_record(struct flb_loki *ctx,
                        msgpack_packer *mp_pck, msgpack_object *rec)
 {
     int i;
+    int skip = 0;
     int len;
+    int ret;
     int size_hint = 1024;
     char *line;
     flb_sds_t buf;
     msgpack_object key;
     msgpack_object val;
+    char *tmp_sbuf_data = NULL;
+    size_t tmp_sbuf_size;
+    msgpack_unpacked mp_buffer;
+    size_t off = 0;
+
+    /* Remove keys in remove_keys */
+    msgpack_unpacked_init(&mp_buffer);
+    if (ctx->remove_mpa) {
+        ret = flb_mp_accessor_keys_remove(ctx->remove_mpa, rec,
+                                          (void *) &tmp_sbuf_data, &tmp_sbuf_size);
+        if (ret == FLB_TRUE) {
+            ret = msgpack_unpack_next(&mp_buffer, tmp_sbuf_data, tmp_sbuf_size, &off);
+            if (ret != MSGPACK_UNPACK_SUCCESS) {
+                flb_free(tmp_sbuf_data);
+                msgpack_unpacked_destroy(&mp_buffer);
+                return -1;
+            }
+            rec = &mp_buffer.data;
+        }
+    }
 
     if (ctx->out_line_format == FLB_LOKI_FMT_JSON) {
         line = flb_msgpack_to_json_str(size_hint, rec);
         if (!line) {
+            if (tmp_sbuf_data) {
+                flb_free(tmp_sbuf_data);
+            }
+            msgpack_unpacked_destroy(&mp_buffer);
             return -1;
         }
         len = strlen(line);
@@ -707,11 +845,19 @@ static int pack_record(struct flb_loki *ctx,
     }
     else if (ctx->out_line_format == FLB_LOKI_FMT_KV) {
         if (rec->type != MSGPACK_OBJECT_MAP) {
+            msgpack_unpacked_destroy(&mp_buffer);
+            if (tmp_sbuf_data) {
+                flb_free(tmp_sbuf_data);
+            }
             return -1;
         }
 
         buf = flb_sds_create_size(size_hint);
         if (!buf) {
+            msgpack_unpacked_destroy(&mp_buffer);
+            if (tmp_sbuf_data) {
+                flb_free(tmp_sbuf_data);
+            }
             return -1;
         }
 
@@ -720,15 +866,16 @@ static int pack_record(struct flb_loki *ctx,
             val = rec->via.map.ptr[i].val;
 
             if (key.type != MSGPACK_OBJECT_STR) {
+                skip++;
                 continue;
             }
 
-            if (i > 0) {
+            if (i > skip) {
                 safe_sds_cat(&buf, " ", 1);
             }
 
-            flb_sds_cat(buf, key.via.str.ptr, key.via.str.size);
-            flb_sds_cat(buf, "=", 1);
+            safe_sds_cat(&buf, key.via.str.ptr, key.via.str.size);
+            safe_sds_cat(&buf, "=", 1);
             pack_format_line_value(buf, &val);
         }
 
@@ -737,6 +884,11 @@ static int pack_record(struct flb_loki *ctx,
         flb_sds_destroy(buf);
     }
 
+    msgpack_unpacked_destroy(&mp_buffer);
+    if (tmp_sbuf_data) {
+        flb_free(tmp_sbuf_data);
+    }
+
     return 0;
 }
 
@@ -829,7 +981,7 @@ static flb_sds_t loki_compose_payload(struct flb_loki *ctx,
          msgpack_pack_str_body(&mp_pck, "stream", 6);
 
          /* Pack stream labels */
-         pack_labels(ctx, &mp_sbuf, &mp_pck, tag, tag_len, obj);
+         pack_labels(ctx, &mp_sbuf, &mp_pck, tag, tag_len, NULL);
 
         /* streams['values'] */
          msgpack_pack_str(&mp_pck, 6);
@@ -1050,6 +1202,12 @@ static struct flb_config_map config_map[] = {
      "Comma separated list of keys to use as stream labels."
     },
 
+    {
+     FLB_CONFIG_MAP_CLIST, "remove_keys", NULL,
+     0, FLB_TRUE, offsetof(struct flb_loki, remove_keys),
+     "Comma separated list of keys to remove."
+    },
+
     {
      FLB_CONFIG_MAP_STR, "line_format", "json",
      0, FLB_TRUE, offsetof(struct flb_loki, line_format),
