@@ -2,7 +2,7 @@
 
 /*  Fluent Bit
  *  ==========
- *  Copyright (C) 2019-2020 The Fluent Bit Authors
+ *  Copyright (C) 2019-2021 The Fluent Bit Authors
  *  Copyright (C) 2015-2018 Treasure Data Inc.
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
@@ -306,6 +306,7 @@ struct mk_list *flb_config_map_create(struct flb_config *config,
         new->set_property = m->set_property;
         new->offset = m->offset;
         new->value.mult = NULL;
+        new->desc = m->desc;
         mk_list_add(&new->_head, list);
 
         if (new->set_property == FLB_FALSE) {
@@ -681,7 +682,7 @@ int flb_config_map_set(struct mk_list *properties, struct mk_list *map, void *co
 
             /* Populate value */
             if (m->type == FLB_CONFIG_MAP_STR) {
-                entry->val.str = kv->val;
+                entry->val.str = flb_sds_create(kv->val);
             }
             else if (m->type == FLB_CONFIG_MAP_INT) {
                 entry->val.i_num = atoi(kv->val);
