@@ -2,7 +2,7 @@
 
 /*  Fluent Bit
  *  ==========
- *  Copyright (C) 2019-2020 The Fluent Bit Authors
+ *  Copyright (C) 2019-2021 The Fluent Bit Authors
  *  Copyright (C) 2015-2018 Treasure Data Inc.
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
@@ -34,6 +34,7 @@
 #include <fluent-bit/flb_metrics.h>
 #include <fluent-bit/flb_storage.h>
 #include <fluent-bit/flb_kv.h>
+#include <fluent-bit/flb_hash.h>
 
 struct flb_libco_in_params libco_in_param;
 
@@ -141,6 +142,13 @@ struct flb_input_instance *flb_input_new(struct flb_config *config,
         /* Get an ID */
         id =  instance_id(plugin, config);
 
+        /* Index for Chunks (hash table) */
+        instance->ht_chunks = flb_hash_create(FLB_HASH_EVICT_NONE, 512, 0);
+        if (!instance->ht_chunks) {
+            flb_free(instance);
+            return NULL;
+        }
+
         /* format name (with instance id) */
         snprintf(instance->name, sizeof(instance->name) - 1,
                  "%s.%i", plugin->name, id);
@@ -171,7 +179,7 @@ struct flb_input_instance *flb_input_new(struct flb_config *config,
         mk_list_init(&instance->tasks);
         mk_list_init(&instance->chunks);
         mk_list_init(&instance->collectors);
-        mk_list_init(&instance->threads);
+        mk_list_init(&instance->coros);
 
         /* Initialize properties list */
         flb_kv_init(&instance->properties);
@@ -186,7 +194,7 @@ struct flb_input_instance *flb_input_new(struct flb_config *config,
         }
 
         /* Plugin requires a Thread context */
-        if (plugin->flags & FLB_INPUT_THREAD) {
+        if (plugin->flags & FLB_INPUT_CORO) {
             instance->threaded = FLB_TRUE;
         }
 
@@ -392,8 +400,14 @@ void flb_input_instance_destroy(struct flb_input_instance *ins)
         flb_config_map_destroy(ins->config_map);
     }
 
+    /* hash table for chunks */
+    if (ins->ht_chunks) {
+        flb_hash_destroy(ins->ht_chunks);
+    }
+
     /* Unlink and release */
     mk_list_del(&ins->_head);
+
     flb_free(ins);
 }
 
@@ -577,7 +591,7 @@ int flb_input_check(struct flb_config *config)
 /*
  * API for Input plugins
  * =====================
- *  Copyright (C) 2019-2020 The Fluent Bit Authors
+ *  Copyright (C) 2019-2021 The Fluent Bit Authors
  * The Input interface provides a certain number of functions that can be
  * used by Input plugins to configure it own behavior and request specific
  *
@@ -955,7 +969,7 @@ int flb_input_collector_fd(flb_pipefd_t fd, struct flb_config *config)
 {
     struct mk_list *head;
     struct flb_input_collector *collector = NULL;
-    struct flb_thread *th;
+    struct flb_coro *co;
 
     mk_list_foreach(head, &config->collectors) {
         collector = mk_list_entry(head, struct flb_input_collector, _head);
@@ -980,11 +994,11 @@ int flb_input_collector_fd(flb_pipefd_t fd, struct flb_config *config)
 
     /* Trigger the collector callback */
     if (collector->instance->threaded == FLB_TRUE) {
-        th = flb_input_thread_collect(collector, config);
-        if (!th) {
+        co = flb_input_coro_collect(collector, config);
+        if (!co) {
             return -1;
         }
-        flb_thread_resume(th);
+        flb_coro_resume(co);
     }
     else {
         collector->cb_collect(collector->instance, config,
