@@ -285,10 +285,80 @@ void cb_array_id()
     msgpack_unpacked_destroy(&result);
 }
 
+void cb_get_kv_pair()
+{
+    int len;
+    int ret;
+    int type;
+    size_t off = 0;
+    char *out_buf;
+    size_t out_size;
+    char *json;
+    char *fmt;
+    char *fmt_out;
+    msgpack_object *start_key;
+    msgpack_object *out_key;
+    msgpack_object *out_val;
+    msgpack_unpacked result;
+    msgpack_object map;
+    struct flb_record_accessor *ra;
+
+    /* Sample JSON message */
+    json =
+        "{\"key1\": \"something\", "
+        "\"kubernetes\": "
+        "   [true, "
+        "    false, "
+        "    {\"a\": false, "
+        "     \"annotations\": { "
+        "                       \"fluentbit.io/tag\": \"thetag\""
+        "}}]}";
+
+    /* Convert to msgpack */
+    len = strlen(json);
+    ret = flb_pack_json(json, len, &out_buf, &out_size, &type);
+    TEST_CHECK(ret == 0);
+    if (ret == -1) {
+        exit(EXIT_FAILURE);
+    }
+
+    /* Formatter */
+    fmt = flb_sds_create("$kubernetes[2]['annotations']['fluentbit.io/tag']");
+    fmt_out = "thetag";
+
+    ra = flb_ra_create(fmt, FLB_FALSE);
+    TEST_CHECK(ra != NULL);
+    if (!ra) {
+        exit(EXIT_FAILURE);
+    }
+
+    /* Unpack msgpack object */
+    msgpack_unpacked_init(&result);
+    msgpack_unpack_next(&result, out_buf, out_size, &off);
+    map = result.data;
+
+    /* Do translation */
+    ret = flb_ra_get_kv_pair(ra, map, &start_key, &out_key, &out_val);
+    TEST_CHECK(ret == 0);
+    if (ret != 0) {
+        exit(EXIT_FAILURE);
+    }
+
+    TEST_CHECK(out_val->type == MSGPACK_OBJECT_STR);
+    TEST_CHECK(out_val->via.str.size == strlen(fmt_out));
+    TEST_CHECK(memcmp(out_val->via.str.ptr, fmt_out, strlen(fmt_out)) == 0);
+
+    flb_sds_destroy(fmt);
+    flb_ra_destroy(ra);
+    flb_free(out_buf);
+    msgpack_unpacked_destroy(&result);
+}
+
 TEST_LIST = {
     { "keys"         , cb_keys},
     { "translate"    , cb_translate},
     { "dots_subkeys" , cb_dots_subkeys},
     { "array_id"     , cb_array_id},
+    { "get_kv_pair"  , cb_get_kv_pair},
     { NULL }
 };
